endPoints = {
    "register": "/register",
    "checkKyc": "/check_kyc",
    "checkHandle": "/check_handle",
    "requestKyc": "/request_kyc",
    "issueSila": "/issue_sila",
    "redeemSila": "/redeem_sila",
    "transferSila": "/transfer_sila",
    "addCrypto": "/add_crypto",
    "addIdentity": "/add_identity",
    "createBond": "/create_bond",
    "checkHandle": "/check_handle",
    "getAccounts": "/get_accounts",
    "getAccountBalance": "/get_account_balance",
    "getTransactions": "/get_transactions",
    "registerOperator": "/register_operator",
    "linkAccount": "/link_account",
    "plaidSameDayAuth": "/plaid_sameday_auth",
    "registerWallet": "/register_wallet",
    "getWallets": "/get_wallets",
    "getWallet": "/get_wallet",
    "updateWallet": "/update_wallet",
    "deleteWallet": "/delete_wallet",
    "getSilaBalance": "/get_sila_balance",
    "schemaUrl": "https://api.silamoney.com/0.2/getmessage?emptymessage=%sTestMessage",
    "apiUrl": "https://%s.silamoney.com/0.2",
    "silaBalanceProd": "https://silatokenapi.silamoney.com/silaBalance",
    "silaBalanceSandbox": "https://sandbox.silatokenapi.silamoney.com/silaBalance",
    "getBusinessTypes": "/get_business_types",
    "getBusinessRoles": "/get_business_roles",
    "getNaicsCategories": "/get_naics_categories",
    "linkBusinessMember": "/link_business_member",
    "unlinkBusinessMember": "/unlink_business_member",
    "getEntities": "/get_entities?",
    "getEntity": "/get_entity",
    "certifyBeneficialOwner": "/certify_beneficial_owner",
    "certifyBusiness": "/certify_business",
    "cancelTransaction": "/cancel_transaction",
    "addRegistrationData": "/add/",
    "updateRegistrationData": "/update/",
    "deleteRegistrationData": "/delete/",
    "documents": "/documents",
    "list_documents": "/list_documents?",
    "get_document": "/get_document",
    "list_supported_documents": "/document_types?",
}



Errors = {
        400: "BAD_REQUEST_BODY",
        401: "UNAUTHORIZED",
        403: "ACCESS_DENIED",
        404: "NotFoundError",
        406: "NotAcceptableError",
        409: "ConflictError",
        415: "UnsupportedMediaTypeError",
        422: "UnprocessableEntityError",
        429: "TooManyRequestsError",
        500: "API_CONFIGURATION_ERROR",
        502: "BadGatewayError",
        503: "ServiceUnavailableError",
        504: "GatewayTimeoutError"
    }


Schema = [
    {"header_msg": {
        "header": {
            "reference": "",
            "created": "",
            "user_handle": "",
            "auth_handle": "",
            "version": "0.2",
            "crypto": "ETH"
        },
        "message": "header_msg",
        "kyc_level": "",
        "entity_type": ""
    }},
    {"link_account_msg_plaid": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": "",
            "version": "0.2",
            "crypto": "ETH",
            "reference": ""
        },
        "message": "link_account_msg",
        "public_token": "",
        "account_name": "",
        "selected_account_id": ""
    }},
    {"link_account_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": "",
            "version": "0.2",
            "crypto": "ETH",
            "reference": ""
        },
        "message": "link_account_msg",
        "account_number": "",
        "routing_number": "",
        "account_type": "",
        "account_name": "",
        "selected_account_id": ""
    }},
    {"account_name_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": "",
        },
        "account_name": ""
    }},
    {"transfer_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": "",
            "version": "0.2",
            "crypto": "ETH",
            "reference": ""
        },
        "message": "transfer_msg",
        "amount": "",
        "destination": "",
        "destination_handle": "",
        "destination_wallet": "",
        "destination_address": "",
        "business_uuid": "",
        "descriptor": ""
    }},
    {"issue_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": "",
            "version": "0.2",
            "crypto": "ETH",
            "reference": ""
        },
        "message": "issue_msg",
        "amount": "",
        "account_name": "",
        "business_uuid": "",
        "descriptor": "",
        "processing_type": ""
    }},
    {"redeem_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": "",
            "version": "0.2",
            "crypto": "ETH",
            "reference": ""
        },
        "message": "redeem_msg",
        "amount": "",
        "account_name": "",
        "business_uuid": "",
        "descriptor": "",
        "processing_type": ""
    }},
    {"no_content_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": "",
            "version": "0.2",
            "crypto": "ETH",
            "reference": ""
        }
    }},
    {"update_wallet_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": "",
            "version": "0.2",
            "crypto": "ETH",
            "reference": ""
        },
        "nickname": "",
        "default": False
    }},
    {"register_wallet_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": "",
            "version": "0.2",
            "crypto": "ETH",
            "reference": ""
        },
        "wallet_verification_signature": "",
        "wallet": {
            "blockchain_address": "",
            "blockchain_network": "ETH",
            "nickname": ""
        }
    }},
    {"get_wallets_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": "",
            "version": "0.2",
            "crypto": "ETH",
            "reference": ""
        },
        "search_filters": {
            "page": 1,
            "per_page": 20,
            "sort_ascending": False,
            "blockchain_network": "ETH",
            "blockchain_address": "",
            "nickname": ""
        }
    }},
    {"get_accounts_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": "",
            "version": "0.2",
            "crypto": "ETH",
            "reference": ""
        },
        "message": "get_accounts_msg"
    }},
    {"sila_balance_msg": {
        "address": ""
    }},
    {"business_types_msg": {
        "header": {
            "created": "",
            "auth_handle": ""
        }
    }},
    {"business_roles_msg": {
        "header": {
            "created": "",
            "auth_handle": ""
        }
    }},
    {"naics_categories_msg": {
        "header": {
            "created": "",
            "auth_handle": ""
        }
    }},
    {"entity_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": "",
            "version": "0.2",
            "crypto": "ETH",
            "reference": ""
        },
        "message": "entity_msg",
        "address": {
            "address_alias": "default",
            "street_address_1": "",
            "city": "",
            "state": "",
            "country": "",
            "postal_code": ""
        },
        "identity": {
            "identity_alias": "SSN",
            "identity_value": ""
        },
        "contact": {
            "contact_alias": "default",
            "phone": "",
            "email": ""
        },
        "crypto_entry": {
            "crypto_alias": "default",
            "crypto_code": "ETH",
            "crypto_address": ""
        },
        "entity": {
            "first_name": "",
            "last_name": "",
            "entity_name": "",
            "birthdate": "",
            "relationship": "",
            "type": "",
            "business_type": "",
            "business_website": "",
            "doing_business_as": "",
            "naics_code": ""
        }
    }},
    {"link_business_member_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": "",
            "business_handle": ""
        },
        "role": "",
        "details": "",
        "ownership_stake": ""
    }},
    {"unlink_business_member_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": "",
            "business_handle": ""
        },
        "role": ""
    }},
    {"get_entity_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": ""
        }
    }},
    {"certify_beneficial_owner_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": "",
            "business_handle": ""
        },
        "member_handle": "",
        "certification_token": ""
    }},
    {"certify_business_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": "",
            "business_handle": ""
        }
    }},
    {'get_transaction_msg': {
        "header": {
            "created": '',
            "auth_handle": "",
            "user_handle": "",
            "version": "",
            "crypto": "",
            "reference": ""
        },
        "message": "get_transactions_msg",
        "search_filters": {
            "transaction_id": "",
            "reference_id": "",
            "show_timelines": '',
            "sort_ascending": '',
            "max_sila_amount": '',
            "min_sila_amount": '',
            "statuses": [],
            "start_epoch": '',
            "end_epoch": '',
            "page": '',
            "per_page": '',
            "transaction_types": []
        }
    }},
    {'cancel_transaction_msg': {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": "",
            "version": "",
            "crypto": "",
            "reference": ""
        },
        "transaction_id": ""
    }},
    {"add_registration_data_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": ""
        },
        "email": "",
        "phone": "",
        "identity_alias": "",
        "identity_value": "",
        "address_alias": "",
        "street_address_1": "",
        "street_address_2": "",
        "city": "",
        "state": "",
        "postal_code": "",
        "country": ""
    }},
    {"update_registration_data_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": ""
        },
        "uuid": "",
        "email": "",
        "phone": "",
        "identity_alias": "",
        "identity_value": "",
        "address_alias": "",
        "street_address_1": "",
        "street_address_2": "",
        "city": "",
        "state": "",
        "postal_code": "",
        "country": ""
    }},
    {"delete_registration_data_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": ""
        },
        "uuid": ""
    }},
    {"documents_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": "",
            "version": "0.2",
            "crypto": "ETH",
            "reference": ""
        },
        "message": "",
        "name": "",
        "filename": "",
        "hash": "",
        "mime_type": "",
        "document_type": "",
        "identity_type": "",
        "description": ""
    }},
    {"list_documents_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": "",
            "version": "",
            "crypto": "",
            "reference": ""
        },
        "message": "",
        "start_date": "",
        "end_date": "",
        "doc_types": "",
        "search": "",
        "sort_by": ""
    }},
    {"get_document_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "user_handle": "",
            "version": "",
            "crypto": "",
            "reference": ""
        },
        "document_id": ""
    }},
    {"document_types_msg": {
        "header": {
            "created": "",
            "auth_handle": "",
            "version": "",
            "crypto": "",
            "reference": ""
        }
    }}
]



from .ethwallet import EthWallet
from .users import User
from .client import App
from .transactions import Transaction
from .wallet import Wallet
from .businessInformation import BusinessInformation
from .businessOperations import BusinessOperations
from .processingTypes import ProcessingTypes
from .registrationFields import RegistrationFields
from .documents import Documents



from .endpoints import endPoints
from silasdk import message


class Documents():
    def uploadDocument(self, payload, file, user_private_key):
        """
        Args:
            payload: document data
            user_private_key
        Returns:
            dict: response body (entity information)
        """
        path = endPoints["documents"]
        msg_type = "documents_msg"
        response = message.postRequest(
            self, path, msg_type, payload, key=user_private_key, content_type="multipart/form-data", fileContents=file)
        return response

    def listDocuments(self, payload, user_private_key, page=None, per_page=None, order=None):
        """
        Args:
            payload: document data
            user_private_key
        Returns:
            dict: response body (entity information)
        """
        path = endPoints["list_documents"] + \
            ('&page=' + str(page) if page is not None else '') + \
            ('&per_page=' + str(per_page) if per_page is not None else '') + \
            ('&order=' + order if order is not None else '')
        msg_type = "list_documents_msg"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def getDocument(self, payload, user_private_key):
        """
        Args:
            payload: document data
            user_private_key
        Returns:
            dict: response body (entity information)
        """
        path = endPoints["get_document"]
        msg_type = "get_document_msg"
        response = message.postGetFile(
            self, path, msg_type, payload, user_private_key)
        if (isinstance(response, dict)):
            return response
        else:
            document_reponse = {
                'status_code': response.status_code,
                'headers': response.headers,
                'content': response.content
            }
            return document_reponse

    def listSupportedDocuments(self, page=None, per_page=None):
        """
        Args:
            payload: document data
            user_private_key
        Returns:
            dict: response body (entity information)
        """
        payload = {}
        path = endPoints["list_supported_documents"] + \
            ('&page=' + str(page) if page is not None else '') + \
            ('&per_page=' + str(per_page) if per_page is not None else '')
        msg_type = "document_types_msg"
        response = message.postRequest(
            self, path, msg_type, payload)
        return response



import requests
import time
import uuid
from copy import deepcopy
from typing import Dict
from .schema import Schema


def createBody(bodyStructure, fields):
    for field in fields:
        if field in bodyStructure.keys():
            bodyStructure[field] = fields[field]
        else:
            for bodyField in bodyStructure:
                if isinstance(bodyStructure[bodyField], dict):
                    createBody(bodyStructure[bodyField], fields)

    return bodyStructure


def getMessage(self, msg_type):
    """gets the message from schema 
    Args:
        path : endpoint path
    """
    for i in Schema:
        for key, value in i.items():
            if key == msg_type:
                return deepcopy(i[msg_type])


def lower_keys(x):
    """converts the payload dict keys to all lowercase to match schema
    Args:
        x:payload
    """
    if isinstance(x, dict):
        return dict((k.lower(), v) for k, v in x.items())
    else:
        return "msg_fromat_incorrect"


def cull_null_values(data: Dict) -> Dict:
    for k, v in list(data.items()):
        if isinstance(v, dict):
            data[k] = cull_null_values(v)
        elif v == '':
            del data[k]

    return data


def createMessage(self, payload, msg_type):
    """creates the message to be sent based on payload from customer
    Args:
        payload:customer message
    """
    payload.update(
        {
            "auth_handle": str(self.app_handle),
            "reference": str(uuid.uuid4()),
            "crypto_code": "ETH",
            "relationship": "user"
        }
    )

    inpt = getMessage(self, msg_type)
    data = lower_keys(payload)

    inpt = createBody(inpt, data)

    try:
        inpt["header"]["created"] = int(time.time())
    except:
        pass

    inpt = cull_null_values(inpt)

    return inpt


def postRequest(self, path, msg_type, payload, key=None, business_key=None, content_type=None, fileContents=None):
    """post the message and return response
    Args:
        payload:customer message
        path : endpoint
        key :user_private_key
    """
    data = createMessage(self, payload, msg_type)
    header = self.setHeader(data, key, business_key, content_type)
    response = self.post(path, data, header) if fileContents is None else self.postFile(
        path, data, header, fileContents)
    return response


def postGetFile(self, path: str, msg_type: str, payload: dict, key: str) -> requests.Response:
    data = createMessage(self, payload, msg_type)
    header = self.setHeader(data, key)
    response = self.postFileResponse(path, data, header)
    return response



from .endpoints import endPoints
from silasdk import message
import json
import requests


class User():
    def checkHandle(self, payload):
        """Check if the user handle is available.
        These endpoint returns the validity of a user handle
        Args:
        payload : Required user_handle to check if its available
        Returns:
        dict: response body (a confirmation message)
        """
        path = endPoints["checkHandle"]
        msg_type = "header_msg"
        response = message.postRequest(self, path, msg_type, payload)
        return response

    def register(self, payload):
        """Register a new user.
           This user will be kyced and ethereum address will be registered with sila 
        Args:
            payload : info about user like name,ssn, dob,ethereum address, ethereum handle etc
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["register"]
        msg_type = "entity_msg"
        response = message.postRequest(self, path, msg_type, payload)
        return response

    def requestKyc(self, payload, user_private_key, use_kyc_level=False):
        """Request kyc for a user by handle
           This user will be kyced and ethereum address will be registered with sila 
        Args:
            payload : info about user like name,ssn, dob,ethereum address, ethereum handle etc
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["requestKyc"]
        msg_type = "header_msg"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def linkAccount(self, payload, user_private_key, plaid=False):
        """link the bank account of user using plad
           This users bank account will be linked  
        Args:
            payload : need user handle and plad token
            user_private_key: users ethereum private key 
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["linkAccount"]
        msg_type = ("link_account_msg" if (plaid is False)
                    else "link_account_msg_plaid")
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def checkKyc(self, payload, user_private_key):
        """check if the user has been kyced.
            The user will be checked if the they have been kyced
        Args:
            payload : includes 
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["checkKyc"]
        msg_type = "header_msg"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def addIdentity(self, payload, user_private_key):
        """change the info about user like change ssn, email ,etc.
            The used will be checked if the they have been kyced
        Args:
            payload : includes information to be edited and usee handle
            user_private_key: users ethereum private key 
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["addIdentity"]
        msg_type = "identity_msg"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def getAccounts(self, payload, user_private_key):
        """get the accounts of users registered with sila
            The user will be checked if they have been kyced, along with app
        Args:
            user_hanlde: users handle registered with app
            user_private_key: user private key asscoicated with crypto address
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["getAccounts"]
        msg_type = "get_accounts_msg"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def getAccountBalance(self, payload, user_private_key):
        """get the account balance of a user registered with sila
        Args:
            user_hanlde: users handle registered with app
            user_private_key: user private key asscoicated with crypto address
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["getAccountBalance"]
        msg_type = "account_name_msg"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def getTransactions(self, payload, user_private_key):
        """get the users transactions registered with ur app
           The user will be checked if they have been kyced, along with app
        Args:
            user_hanlde: users handle registered with app
            user_private_key: user private key asscoicated with crypto address
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["getTransactions"]
        msg_type = "get_transaction_msg"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def silaBalance(self, address):
        """get sila balance of the addresses registered with sila
           The user will be checked if they have been kyced, along with app
        Args:
            address: requires valid ethereum address
        Returns:
            dict: response body (a confirmation message)
        """
        data = json.dumps({"address": str(address)})
        header = {'content-type': 'application/json'}
        if self.tier == "prod":
            endpoint = endPoints["silaBalanceProd"]
        elif self.tier == "sandbox":
            endpoint = endPoints["silaBalanceSandbox"]
        response = requests.post(endpoint, data=data, headers=header)
        return response.json()

    def getSilaBalance(self, address):
        """get sila balance of the addresses registered with sila
           The user will be checked if they have been kyced, along with app
        Args:
            address: requires valid ethereum address
        Returns:
            dict: response body (a confirmation message)
        """
        payload = {"address": str(address)}
        header = {'content-type': 'application/json'}
        path = endPoints["getSilaBalance"]
        msg_type = "sila_balance_msg"
        response = message.postRequest(
            self, path, msg_type, payload)
        return response

    def getEntities(self, payload, per_page=None, page=None):
        """Return all end-user and legal entities (businesses) associated with a customer application.
            This endpoint allows the listing of all entities registered to an application.
        Args:
            payload: filters information
            per_page: pagination information
            page: pagination information
        Returns:
            dict: response body (entities list)
        """
        path = endPoints["getEntities"] + (('&per_page=' + str(per_page)) if per_page is not None else '') + (('&page=' + str(page)) if page is not None else '') 
        msg_type = "header_msg"
        response = message.postRequest(
            self, path, msg_type, payload)
        return response

    def getEntity(self, payload, user_private_key):
        """
        Args:
            payload: filters information
            user_private_key
        Returns:
            dict: response body (entity information)
        """
        path = endPoints["getEntity"]
        msg_type = "get_entity_msg"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def addRegistrationData(self, registrationField, payload, user_private_key):
        """
        Args:
            payload: registration data
            user_private_key
        Returns:
            dict: response body (entity information)
        """
        path = endPoints["addRegistrationData"] + registrationField
        msg_type = "add_registration_data_msg"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def updateRegistrationData(self, registrationField, payload, user_private_key):
        """
        Args:
            payload: registration data
            user_private_key
        Returns:
            dict: response body (entity information)
        """
        path = endPoints["updateRegistrationData"] + registrationField
        msg_type = "update_registration_data_msg"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response

    def deleteRegistrationData(self, registrationField, payload, user_private_key):
        """
        Args:
            payload: registration data
            user_private_key
        Returns:
            dict: response body (entity information)
        """
        path = endPoints["deleteRegistrationData"] + registrationField
        msg_type = "delete_registration_data_msg"
        response = message.postRequest(
            self, path, msg_type, payload, user_private_key)
        return response


from .endpoints import endPoints
from .client import App
from silasdk import message


class BusinessInformation():

    def getBusinessTypes(self):
        """Gets a list of valid business types that can be registered.
        Args:
        Returns:
            dict: response body (business types)
        """
        path = endPoints["getBusinessTypes"]
        msg_type = "business_types_msg"
        payload = {}
        response = message.postRequest(self, path, msg_type, payload)
        return response

    def getBusinessRoles(self):
        """Retrieves the list of pre-defined business roles.
        Args:
        Returns:
            dict: response body (business roles)
        """
        path = endPoints["getBusinessRoles"]
        msg_type = "business_roles_msg"
        payload = {}
        response = message.postRequest(self, path, msg_type, payload)
        return response

    def getNaicsCategories(self):
        """
        Args:
        Returns:
            dict: response body (business roles)
        """
        path = endPoints["getNaicsCategories"]
        msg_type = "naics_categories_msg"
        payload = {}
        response = message.postRequest(self, path, msg_type, payload)
        return response



from .endpoints import endPoints
from .client import App
from silasdk import message


class Transaction():

    def issueSila(self, payload, user_private_key):
        """issues sila erc20token for dollar amount on ethereum blockchain to kyced ethereum addresses (price one cent per token)
            the handle address signatures need to be verified
        Args:
            payload : includes user handle and amount
            user_private_key: users ethereum private key 
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["issueSila"]
        msg_type = "issue_msg"
        response = message.postRequest(self, path, msg_type, payload, user_private_key)
        return response

    def redeemSila(self, payload, user_private_key):
        """redeems sila erc20token for dollar amount on ethereum blockchain to kyced ethereum addresses (price one cent per token)
            the handle address signatures need to be verified
        Args:
            payload : user handle and amount
            user_private_key: users ethereum private key 
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["redeemSila"]
        msg_type = "redeem_msg"
        response = message.postRequest(self, path, msg_type, payload, user_private_key)
        return response

    def transferSila(self, payload, user_private_key, use_destination_address=False):
        """ transfer sila from one ethereum address to another using sila api
            the handle address signatures need to be verified
        Args:
            payload : user handle and amount
            user_private_key: users ethereum private key 
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["transferSila"]
        msg_type = "transfer_msg"
        response = message.postRequest(self, path, msg_type, payload, user_private_key)
        return response

    def plaidSamedayAuth(self, payload, user_private_key):
        """ Handle a request for a Plaid public_token in order to complete
            Plaid's Same Day Microdeposit Authentication
        Args:
            payload : user handle and amount
            user_private_key: users ethereum private key
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["plaidSameDayAuth"]
        msg_type = "account_name_msg"
        response = message.postRequest(self, path, msg_type, payload, user_private_key)
        return response

    def cancelTransaction(self, payload, user_private_key):
        """ Cancel a pending transaction under certain circumstances


        Args:
            payload : user handle and transaction id
            user_private_key: users ethereum private key
        Returns:
            dict: response body (a confirmation message)
        """
        path = endPoints["cancelTransaction"]
        msg_type = "cancel_transaction_msg"
        response = message.postRequest(self, path, msg_type, payload, user_private_key)
        return response



import json
import requests
from .ethwallet import EthWallet
from .endpoints import endPoints
from .errors import Errors
from .schema import Schema


class App():

    def __init__(self, tier, app_private_key, app_handle):
        """Initalize the application 
            This lets users initialize the application by providing the tier, application privatekey and application handle
        Args:
            tier  : SANDBOX,PROD etc
            app_private_key : ethereum privat key for the application
            app_handle  : application sila handle (app.silamoney.eth)
        """
        self.session = requests.Session()
        self.tier = tier.lower()
        self.app_private_key = app_private_key
        self.app_handle = app_handle
        self.updateSchema()

    def updateSchema(self):
        """updates schema.py on initialization of app
            This lets users initialize the schema into schema.py for ease of use
        Args:
            None
        """
        endpoint = endPoints["schemaUrl"]
        message = ["header", "entity", "identity", "crypto", "linkAccount"]
        for i in message:
            response = self.get(
                endpoint % i)
            sch = {response["message"]: response}
            Schema.append(sch)

    def getUrl(self):
        """construct the url endpoint to make api calls
        Args:
            app: the initialized applications
        """
        url = endPoints["apiUrl"]
        if self.tier == "prod":
            apiurl = url % "api"
        else:
            apiurl = url % self.tier
        return apiurl

    def post(self, path, payload, header):
        """makes a post request to the sila_apis
        Args:
            path : path to the endpoint being called
            payload : json msg to be posted 
            header  : contains the usersignature and authsignature
        """
        url = self.getUrl()
        endpoint = url + path
        data1 = json.dumps(payload)
        response = self.session.post(
            endpoint,
            data=data1,
            headers=header)

        output = response.json()

        try:
            output['status_code'] = response.status_code
        except:
            pass

        return output

    def postFile(self, path, payload, header, fileContents):
        url = self.getUrl()
        endpoint = url + path
        message = json.dumps(payload)
        files = {'file': fileContents}
        response = requests.post(
            endpoint,
            data={'data': message},
            headers=header,
            files=files
        )

        output = response.json()

        return output

    def postFileResponse(self, path: str, payload: dict, header: dict) -> requests.Response:
        url = self.getUrl()
        endpoint = url + path
        data = json.dumps(payload)
        response = self.session.post(
            endpoint,
            data=data,
            headers=header
        )

        if (response.status_code == 200):
            return response
        else:
            return response.json()

    def postPlaid(self, url, payload):
        """makes a post request to the sila_apis
        Args:
            path : path to the endpoint being called
            payload : json msg to be posted
            header  : contains the usersignature and authsignature
        """
        content = json.dumps(payload)
        response = self.session.post(
            url,
            data=content,
            headers={
                "Content-Type": "application/json"
            })
        output = response.json()
        return output

    def get(self, path):
        """make a get request using this function
        Args:
            path : path to the endpoint
        """
        endpoint = path
        response = self.session.get(endpoint)
        output = response.json()
        return output

    def setHeader(self, msg, key=None, business_key=None, content_type=None):
        """set the application header with usersignature and authsignature
        Args:
            key : ethereum private key for the user
            msg : message being sent should be signed by user
        """
        appsignature = EthWallet.signMessage(msg, self.app_private_key)
        header = {
            "authsignature": appsignature
        }
        if content_type is not None and content_type == 'multipart/form-data':
            pass
        else:
            header["Content-Type"]: 'application/json' if content_type is None else content_type
        if key is not None:
            header["usersignature"] = EthWallet.signMessage(msg, key.lower())
        if business_key is not None:
            header["businesssignature"] = EthWallet.signMessage(
                msg, business_key.lower())

        return header



from .endpoints import endPoints
from .client import App
from silasdk import message


class BusinessOperations():

    def linkBusinessMember(self, payload, user_private_key, business_private_key):
        """
        Args:
            payload: includes member information.
            user_private_key:
            business_private_key:
        Returns:
            dict: response body (confirmation message)
        """
        path = endPoints["linkBusinessMember"]
        msg_type = "link_business_member_msg"
        response = message.postRequest(self, path, msg_type, payload, user_private_key, business_private_key)
        return response

    def unlinkBusinessMember(self, payload, user_private_key, business_private_key):
        """
        Args:
            payload: includes member information.
            user_private_key:
            business_private_key:
        Returns:
            dict: response body (confirmation message)
        """
        path = endPoints["unlinkBusinessMember"]
        msg_type = "unlink_business_member_msg"
        response = message.postRequest(self, path, msg_type, payload, user_private_key, business_private_key)
        return response

    def certifyBeneficialOwner(self, payload, user_private_key, business_private_key):
        """
        Args:
            payload: includes member information.
            user_private_key:
            business_private_key:
        Returns:
            dict: response body (confirmation message)
        """
        path = endPoints["certifyBeneficialOwner"]
        msg_type = "certify_beneficial_owner_msg"
        response = message.postRequest(self, path, msg_type, payload, user_private_key, business_private_key)
        return response

    def certifyBusiness(self, payload, user_private_key, business_private_key):
        """
        Args:
            payload: includes business information.
            user_private_key:
            business_private_key:
        Returns:
            dict: response body (confirmation message)
        """
        path = endPoints["certifyBusiness"]
        msg_type = "certify_business_msg"
        response = message.postRequest(self, path, msg_type, payload, user_private_key, business_private_key)
        return response



import setuptools
from setuptools import setup, find_packages
from codecs import open
from os import path

setup(
    name='silasdk',


    version='0.2.13.rc',

    description='Sila Python library for message signing and api wrapper',

    url="https://github.com/Sila-Money/Sila-Python",

    author='Sila',
    author_email='support@silamoney.com',

    license='Apache',

    classifiers=[
        
        # How stable is your project? 
        #   3 - Alpha, 4 - Beta,5 - Production/Stable

        'Development Status :: 4 - Beta',

        'Intended Audience :: Developers',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 3.6'

    ],

    keywords='Sila v0.2.13.rc Rest API',

    packages=["silasdk"],

    install_requires=["requests>=2.20.0","pyaml>=15.8.2","eth-account==0.3.0","uuid==1.30","pysha3==1.0.2"],

    zip_safe=False,
    
    include_package_data=True,

    extras_require={},

   
    package_data={},

  
    entry_points={}
)


from eth_account import Account
import sha3, _pysha3
import json

from typing import Dict, Union


class EthWallet:

    @staticmethod
    def create(entropy=''):
        """Create an Ethereum wallet for a user.

        This will generate a private key and ethereum address, that can be used for trasaction,
        however this not a recommended way to create your wallets
        Args:
        entropy : provide randomness to gnerate the wallet
        Returns:
        tuple: response body with ethereum address and private key
        """
        account = Account.create(entropy)
        return {"eth_private_key": account.privateKey.hex(), "eth_address": account.address}

    @staticmethod
    def signMessage(msg: Union[str, Dict], key=None):
        """Sign the message using an Ethereum private key.

        This method signs the message for the user authentication mechanism
        Args:
        msg: message to be signed
        private_key: the key can be an app key or a user key used to sign the message
        Returns:
        string: a signed message
        """
        k = _pysha3.keccak_256()
        if isinstance(msg, str):
            encoded_message = msg.encode('utf-8')
        else:
            encoded_message = (json.dumps(msg)).encode("utf-8")
        k.update(encoded_message)
        message_hash = k.hexdigest()
        if key is not None:
            signed_message = Account.signHash(message_hash, key)
            sig_hx = signed_message.signature.hex()
            return str(sig_hx.replace("0x", ""))

        else:
            return " "

    @staticmethod
    def verifySignature(msg: Union[str, Dict], sign):
        """Verify the message signature.

        This method signs the message for the user authentication mechanism
        Args:
        msg: original message
        sign : the signed hash obtained after signing the message with private key
        Returns:
        string: returns the Ethereum address corresponding to the private key the message was signed with
        """
        k = _pysha3.keccak_256()
        if isinstance(msg, str):
            encoded_message = msg.encode('utf-8')
        else:
            encoded_message = (json.dumps(msg)).encode("utf-8")
        k.update(encoded_message)
        message_hash = k.hexdigest()
        return Account.recoverHash(message_hash, signature=sign)



from .endpoints import endPoints
from silasdk import message
import json
import requests

class Wallet():
    def registerWallet(self, payload, user_private_key):
        """Register a new wallet
        Args:
        payload : Required user_handle to check if its available
        Returns:
        dict: response body (a confirmation message)
        """
        path = endPoints["registerWallet"]
        msg_type = "register_wallet_msg"
        response = message.postRequest(self, path, msg_type, payload, user_private_key)
        return response

    def getWallets(self, payload, user_private_key):
        """Get wallet list
        Args:
        payload : Required user_handle to check if its available
        Returns:
        dict: response body (a confirmation message)
        """
        path = endPoints["getWallets"]
        msg_type = "get_wallets_msg"
        response = message.postRequest(self, path, msg_type, payload, user_private_key)
        return response

    def getWallet(self, payload, user_private_key):
        """Get a user wallet
        Args:
        payload : Required user_handle to check if its available
        Returns:
        dict: response body (a confirmation message)
        """
        path = endPoints["getWallet"]
        msg_type = "no_content_msg"
        response = message.postRequest(self, path, msg_type, payload, user_private_key)
        return response

    def updateWallet(self, payload, user_private_key):
        """Update a wallet
        Args:
        payload : Required user_handle to check if its available
        Returns:
        dict: response body (a confirmation message)
        """
        path = endPoints["updateWallet"]
        msg_type = "update_wallet_msg"
        response = message.postRequest(self, path, msg_type, payload, user_private_key)
        return response

    def deleteWallet(self, payload, user_private_key):
        """Delete a wallet
        Args:
        payload : Required user_handle to check if its available
        Returns:
        dict: response body (a confirmation message)
        """
        path = endPoints["deleteWallet"]
        msg_type = "no_content_msg"
        response = message.postRequest(self, path, msg_type, payload, user_private_key)
        return response